// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * RWTHonline events AMD module.
 *
 * @module     local_rwth_sd_sync/RWTHonlineEvents
 * @copyright  2018 Tim Schroeder <t.schroeder@itc.rwth-aachen.de>
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

define([
    'jquery',
    'core/ajax',
    'core/notification',
    'core/str',
    'core/templates',
    'core_calendar/events',
    'theme_boost/bootstrap/tooltip'
], function(
    $,
    Ajax,
    Notification,
    Str,
    Templates,
    CalendarEvents
) {

    var cacheGetRenderAsRWTHonlineEvent = function(eventid) {
        var cache = localStorage.getItem('local_rwth_sd_sync/is_rendered_as_rwthonline_event');
        if (cache === null) {
            cache = [];
        } else {
            cache = JSON.parse(cache);
        }
        return cache[eventid];
    };

    var cacheSetRenderAsRWTHonlineEvent = function(eventid, value) {
        var cache = localStorage.getItem('local_rwth_sd_sync/is_rendered_as_rwthonline_event');
        if (cache === null) {
            cache = {};
        } else {
            cache = JSON.parse(cache);
        }
        cache[eventid] = value;
        localStorage.setItem('local_rwth_sd_sync/is_rendered_as_rwthonline_event', JSON.stringify(cache));
    };

    var ifRenderAsRWTHonlineEvent = function(eventid, then) {
        var cachedValue = cacheGetRenderAsRWTHonlineEvent(eventid);
        if (cachedValue != undefined) {
            // We already queried this event.
            if (cachedValue == true) {
                then();
            }
            return;
        }

        // Event has not been queried.
        Ajax.call([
            {methodname: 'local_rwth_sd_sync_is_rendered_as_rwthonline_event', args: {
                id: eventid
            }}
        ])[0].done(function(renderAsRWTHonlineEvent) {
            if (renderAsRWTHonlineEvent) {
                then();
            }
            cacheSetRenderAsRWTHonlineEvent(eventid, renderAsRWTHonlineEvent);
        }).fail(Notification.exception);
    };

    // Remove edit/delete buttons from modals in month view.
    var fixModal = function() {
        var targetNode = document.getElementById('page-calendar-view');
        if (!targetNode) {
            return;
        }
        var config = { childList: true };
        var callback = function(mutationsList) {
            mutationsList.forEach(function(mutation) {
                if (mutation == 0) {
                    return true;
                }
                mutation.addedNodes.forEach(function(node) {
                    if (node.nodeName == "DIV" && node.innerHTML == "" && node.childNodes.length == 0) {
                        observer.observe(node, {childList: true});
                        return true;
                    }
                    var dialog = node.classList.contains('summary-modal-container') ?
                        document.querySelector('#page-calendar-view .modal-dialog') : node.querySelector('.modal-dialog');
                    var summaryContainer = node.classList.contains('summary-modal-container') ?
                        node : node.querySelector('.modal-body .summary-modal-container');

                    if (!dialog && !summaryContainer) {
                        // Not a modal dialog.
                        return true;
                    }

                    if (dialog && !summaryContainer) {
                        // Modal body is loaded but content still loading, await insertion of content block
                        observer.observe(node.querySelector('.modal-body'), {childList: true});
                        return true;
                    }

                    var eventid = summaryContainer.getAttribute('data-event-id');
                    ifRenderAsRWTHonlineEvent(eventid, function() {

                        // Do not render twice
                        var footer = dialog.querySelector('.modal-footer');
                        if (footer.querySelector(".rwthonline_event_info_rendered")) {
                            // Already rendered
                            return true;
                        }

                        Str.get_string('rwthonline_event_help', 'local_rwth_sd_sync').done(function(localizedString) {
                            Ajax.call([{
                                methodname: 'local_rwth_sd_sync_get_rwthonline_event_url',
                                args: {
                                    id: eventid
                                },
                                done: function(rwthonlineurl) {
                                    Templates.render('local_rwth_sd_sync/rwthonline_event_info_modal',
                                        {rwthonlineurl: rwthonlineurl, text: localizedString})
                                    .done(function(rwthonlineInfo) {
                                        footer.innerHTML = rwthonlineInfo;
                                        footer.classList.remove("hidden");
                                    });
                                },
                                fail: function(ex) {
                                    Notification.exception(ex);
                                },
                            }]);
                        });
                    });
                });
            });
        };
        var observer = new MutationObserver(callback);
        observer.observe(targetNode, config);
    };

    // Mark RWTHonline events as such.
    var fixMonthView = function() {
        $('[data-region="event-item"]').each(function(evtIdx, event) {
            var link = $(event).find('a[data-action="view-event"]');
            var title = link.attr('title');
            link.attr('title', $('<textarea />').html(title).text()); // JS equivalent of PHP html_entity_decode
        });
    };

    // Mark RWTHonline events as such and remove edit/delete buttons from day view.
    var fixDayView = function() {
        $(".eventlist .event").each(function(evtIdx, event) {
            var eventid = $(event).attr('data-event-id');

            // Check if the event shoud be rendered as a RWTHonline event.
            ifRenderAsRWTHonlineEvent(eventid, function() {

                // Replace command buttons with text.
                var commands = $(event).find('.commands');
                if (commands.find(".rwthonline_event_info_rendered").length) {
                    // Command buttons already replaced.
                    return;
                }
                Str.get_string('rwthonline_event_help', 'local_rwth_sd_sync').done(function(localizedString) {
                    Ajax.call([
                        {methodname: 'local_rwth_sd_sync_get_rwthonline_event_url', args: {
                            id: eventid
                        }}
                    ])[0].done(function(rwthonlineurl) {
                        Templates.render('local_rwth_sd_sync/rwthonline_event_info_day',
                                {rwthonlineurl: rwthonlineurl, text: localizedString}).done(function(rwthonlineInfo) {
                            commands.empty().append(rwthonlineInfo);
                        });
                    }).fail(Notification.exception);
                });
            });
        });
    };

    // Prevent drag-and-drop.
    var preventDrag = function() {
        var rwthonlineEvents = $("[data-eventtype-rwthonline='1']");
        rwthonlineEvents.on("dragstart", function(event) {
            event.stopImmediatePropagation();
            var target = $(event.target);

            Str.get_string('rwthonline_event_tooltip_drag', 'local_rwth_sd_sync').done(function(localizedString) {

                // Store title in another attribute.
                var title = target.attr('title');
                target.attr('actual-original-title', title);

                // Show tooltip.
                target.attr('title', localizedString);
                target.tooltip({
                    container: "body",
                    trigger: "manual",
                    html: true,
                    template: '<div class="tooltip error" role="tooltip">'
                        + '<div class="tooltip-arrow"></div><div class="tooltip-inner"></div></div>'
                });
                target.tooltip('enable');
                target.tooltip('show');
            });

        });
        rwthonlineEvents.on("dragend", function(event) {
            event.stopImmediatePropagation();
            var target = $(event.target);

            // Delay hiding the tootip slightly.
            setTimeout(function() {
                // Hide tooltip.
                target.tooltip('hide');
                target.tooltip('disable');

                // Restore title.
                var title = target.attr('actual-original-title');
                target.attr('title', title);
            }, 300);
        });
    };

    return {
        init: function() {

            // Load Boost tooltips.
            require(['theme_boost/loader']);

            $(document).ready(function() {
                fixModal();
                preventDrag();
            });
            $('body').on(CalendarEvents.viewUpdated, function() {
                fixMonthView();
                fixDayView();
                preventDrag();
            });
        }
    };
});