<?php
// This file is part of Moodle - http://moodle.org/
// 
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * Version information
 *
 * @package   local_rwth_participants_view
 * @copyright 2018 RWTH Aachen University
 * @author    t.schroeder@itc.rwth-aachen.de
 * @license   http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

defined('MOODLE_INTERNAL') || die;

require_once($CFG->dirroot.'/enrol/externallib.php');

use tool_sdapi\sitedirectory_api;
use tool_sdapi\course_not_found_exception;

class local_rwth_sd_sync_external extends external_api {

    /**
     * Returns description of method parameters value
     *
     * @return external_description
     */
    public static function is_rendered_as_rwthonline_event_parameters() {
        return new external_function_parameters(
            [
                'id' => new external_value(PARAM_INT, 'event id')
            ]
        );
    }

    /**
     * Determine whether the event should be rendered as a RWTHonline event.
     *
     * @param int $eventid Event id
     * @return boolean whether event should be rendered as a RWTHonline event
     */
    public static function is_rendered_as_rwthonline_event($id) {
        global $DB;
        $isrwthonlineevent = $DB->record_exists('rwth_sd_sync_appointment', ['moodle_eventid' => $id, 'eventsource' => 'rwthonline']);
        if (!$isrwthonlineevent) {
            // Is not a RWTHonline event.
            return false;
        }
        $courseid = $DB->get_field('event', 'courseid', ['id' => $id], IGNORE_MISSING);
        if (!$courseid || $courseid == 0) {
            // Event doesn't exist in DB.
            return false;
        }

        // Event should only be rendered as a RWTHonline event if the user can even manage
        // course events created by other users.
        $context = context_course::instance($courseid);
        return has_capability('moodle/calendar:manageentries', $context);
    }

    /**
     * Returns description of method result value
     *
     * @return external_description
     */
    public static function is_rendered_as_rwthonline_event_returns() {
        return new external_value(PARAM_BOOL, 'Whether the event should be rendered as a RWTHonline event');
    }

    /**
     * Returns description of method parameters value
     *
     * @return external_description
     */
    public static function get_rwthonline_event_url_parameters() {
        return new external_function_parameters(
            [
                'id' => new external_value(PARAM_INT, 'event id')
            ]
        );
    }

    /**
     * Return an URL to a page in RWTHonline where you can edit the event with the given id.
     *
     * @param int $eventid Event id
     * @return string RWTHonline URL for editing the event
     */
    public static function get_rwthonline_event_url($id) {
        global $DB;
        $courseid = $DB->get_field('event', 'courseid', ['id' => $id]);
        $idnumber = $DB->get_field('course', 'idnumber', ['id' => $courseid]);

        $sdapi = new sitedirectory_api();
        $sdcourse = null;
        try {
            $sdcourse = $sdapi->get_course($idnumber);
        } catch (course_not_found_exception $e) {
            // Course doesn't exist in SiteDirectory, just use general RWTHonline URL.
            return 'https://online.rwth-aachen.de/';
        }
        $stpspnr = $sdcourse->ron_stp_sp_nr;
        $url = new moodle_url('https://online.rwth-aachen.de/RWTHonline/pl/ui/$ctx/wbTermin_List.wbLehrveranstaltung', ['pStpSpNr' => $stpspnr]);
        return $url->out(false);
    }

    /**
     * Returns description of method result value
     *
     * @return external_description
     */
    public static function get_rwthonline_event_url_returns() {
        return new external_value(PARAM_URL, 'The URL of a page in RWTHonline where you can edit the event');
    }
}