<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * External database sync tests.
 *
 * @package    local_rwth_sd_sync based on enrol_database
 * @category   phpunit
 * @copyright  2011 Petr Skoda {@link http://skodak.org}
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

defined('MOODLE_INTERNAL') || die();

class local_rwth_sd_sync_testcase extends advanced_testcase {
    protected static $courses = array();
    protected static $appointments = array();

    /** @var string Original error log */
    protected $oldlog;

    protected function init_local_rwth_sd_sync() {
        global $DB, $CFG;

        // Discard error logs from AdoDB.
        $this->oldlog = ini_get('error_log');
        ini_set('error_log', "$CFG->dataroot/testlog.log");

        $dbman = $DB->get_manager();

        set_config('dbencoding', 'utf-8', 'local_rwth_sd_sync');

        set_config('dbhost', $CFG->dbhost, 'local_rwth_sd_sync');
        set_config('dbuser', $CFG->dbuser, 'local_rwth_sd_sync');
        set_config('dbpass', $CFG->dbpass, 'local_rwth_sd_sync');
        set_config('dbname', $CFG->dbname, 'local_rwth_sd_sync');

        if (!empty($CFG->dboptions['dbport'])) {
            set_config('dbhost', $CFG->dbhost.':'.$CFG->dboptions['dbport'], 'local_rwth_sd_sync');
        }

        switch ($DB->get_dbfamily()) {

            case 'mysql':
                set_config('dbtype', 'mysqli', 'local_rwth_sd_sync');
                set_config('dbsetupsql', "SET NAMES 'UTF-8'", 'local_rwth_sd_sync');
                set_config('dbsybasequoting', '0', 'local_rwth_sd_sync');
                if (!empty($CFG->dboptions['dbsocket'])) {
                    $dbsocket = $CFG->dboptions['dbsocket'];
                    if ((strpos($dbsocket, '/') === false and strpos($dbsocket, '\\') === false)) {
                        $dbsocket = ini_get('mysqli.default_socket');
                    }
                    set_config('dbtype', 'mysqli://'.rawurlencode($CFG->dbuser).':'.rawurlencode($CFG->dbpass).'@'.rawurlencode($CFG->dbhost).'/'.rawurlencode($CFG->dbname).'?socket='.rawurlencode($dbsocket), 'local_rwth_sd_sync');
                }
                break;

            case 'oracle':
                set_config('dbtype', 'oci8po', 'local_rwth_sd_sync');
                set_config('dbsybasequoting', '1', 'local_rwth_sd_sync');
                break;

            case 'postgres':
                set_config('dbtype', 'postgres7', 'local_rwth_sd_sync');
                $setupsql = "SET NAMES 'UTF-8'";
                if (!empty($CFG->dboptions['dbschema'])) {
                    $setupsql .= "; SET search_path = '".$CFG->dboptions['dbschema']."'";
                }
                set_config('dbsetupsql', $setupsql, 'local_rwth_sd_sync');
                set_config('dbsybasequoting', '0', 'local_rwth_sd_sync');
                if (!empty($CFG->dboptions['dbsocket']) and ($CFG->dbhost === 'localhost' or $CFG->dbhost === '127.0.0.1')) {
                    if (strpos($CFG->dboptions['dbsocket'], '/') !== false) {
                        $socket = $CFG->dboptions['dbsocket'];
                        if (!empty($CFG->dboptions['dbport'])) {
                            $socket .= ':' . $CFG->dboptions['dbport'];
                        }
                        set_config('dbhost', $socket, 'local_rwth_sd_sync');
                    } else {
                      set_config('dbhost', '', 'local_rwth_sd_sync');
                    }
                }
                break;

            case 'mssql':
                if (get_class($DB) == 'mssql_native_moodle_database') {
                    set_config('dbtype', 'mssql_n', 'local_rwth_sd_sync');
                } else {
                    set_config('dbtype', 'mssqlnative', 'local_rwth_sd_sync');
                }
                set_config('dbsybasequoting', '1', 'local_rwth_sd_sync');
                break;

            default:
                throw new exception('Unknown database driver '.get_class($DB));
        }

        // NOTE: It is stongly discouraged to create new tables in advanced_testcase classes,
        //       but there is no other simple way to test ext database enrol sync, so let's
        //       disable transactions are try to cleanup after the tests.

        $table = new xmldb_table('local_rwth_sd_sync_testtrmn');
        $table->add_field('id', XMLDB_TYPE_INTEGER, '10', null, XMLDB_NOTNULL, XMLDB_SEQUENCE, null);
        $table->add_field('courseid', XMLDB_TYPE_CHAR, '255', null, null, null);
        $table->add_field('room', XMLDB_TYPE_CHAR, '255', null, null, null);
        $table->add_field('start', XMLDB_TYPE_DATETIME, null, null, null, null);
        $table->add_field('end', XMLDB_TYPE_DATETIME, null, null, null, null);
        $table->add_field('keyword', XMLDB_TYPE_CHAR, '255', null, null, null);
        $table->add_field('notes', XMLDB_TYPE_CHAR, '255', null, null, null);
        $table->add_key('primary', XMLDB_KEY_PRIMARY, array('id'));
        if ($dbman->table_exists($table)) {
            $dbman->drop_table($table);
        }
        $dbman->create_table($table);
        set_config('appointmenttable', $CFG->prefix.'local_rwth_sd_sync_testtrmn', 'local_rwth_sd_sync');
        set_config('appointmentcoursefield', 'courseid', 'local_rwth_sd_sync');
        set_config('appointmentroomfield', 'room', 'local_rwth_sd_sync');
        set_config('appointmentstartfield', 'start', 'local_rwth_sd_sync');
        set_config('appointmentendfield', 'end', 'local_rwth_sd_sync');
        set_config('appointmentkeywordfield', 'keyword', 'local_rwth_sd_sync');
        set_config('appointmentnotesfield', 'notes', 'local_rwth_sd_sync');
        
        $table = new xmldb_table('local_rwth_sd_sync_testkurs');
        $table->add_field('id', XMLDB_TYPE_INTEGER, '10', null, XMLDB_NOTNULL, XMLDB_SEQUENCE, null);
        $table->add_field('fullname', XMLDB_TYPE_CHAR, '255', null, null, null);
        $table->add_field('shortname', XMLDB_TYPE_CHAR, '255', null, null, null);
        $table->add_field('idnumber', XMLDB_TYPE_CHAR, '255', null, null, null);
        $table->add_field('category', XMLDB_TYPE_CHAR, '255', null, null, null);
        $table->add_field('description', XMLDB_TYPE_CHAR, '255', null, null, null);
        $table->add_key('primary', XMLDB_KEY_PRIMARY, array('id'));
        if ($dbman->table_exists($table)) {
            $dbman->drop_table($table);
        }
        $dbman->create_table($table);
        set_config('newcoursetable', $CFG->prefix.'local_rwth_sd_sync_testkurs', 'local_rwth_sd_sync');
        set_config('newcoursefullname', 'fullname', 'local_rwth_sd_sync');
        set_config('newcourseshortname', 'shortname', 'local_rwth_sd_sync');
        set_config('newcourseidnumber', 'idnumber', 'local_rwth_sd_sync');
        set_config('newcoursecategory', 'category', 'local_rwth_sd_sync');
        set_config('newcoursedescription', 'description', 'local_rwth_sd_sync');
        
        // Create some test users and courses.
        for ($i = 1; $i <= 4; $i++) {
            self::$courses[$i] = $this->getDataGenerator()->create_course(array('fullname' => 'Test course '.$i, 'shortname' => 'tc'.$i, 'idnumber' => 'courseid'.$i, 'description' => 'description'.$i));
        }

        date_default_timezone_set("Europe/Berlin");
        
        //$starttime = time();
        $starttime = strtotime('2018-01-19 08:00:00');
        $duration = 3600;// 1 hour
        
        // @TODO: event creation needs elevated rights -> set user to admin
        self::setUser(2);
        
        // alle in course 1: 'courseid' => 'courseid1'
        $course1 = self::$courses[1];
        //var_dump($course1);
        $course1id = $course1->id;
        
        for ($i = 1; $i <= 10; $i++) {
            self::$appointments[$i] = $this->getDataGenerator()->create_event(array('name' => 'eventname'.$i, 'eventtype' => 'course', 'courseid' => $course1id, 'timestart' => $starttime, 'timeduration' => $duration, 'description' => 'room'.$i));
            
            /*
            $record->name = 'event name';
            $record->eventtype = 'course';
            $record->repeat = 0;
            $record->repeats = 0;
            $record->timestart = time();
            $record->timeduration = 0;
            $record->timesort = 0;
            $record->eventtype = 'user';# should be groupid ? description not set either ?
            $record->courseid = 0;
            $record->categoryid = 0;#not for course type
            */
        }
    }

    protected function cleanup_local_rwth_sd_sync() {
        global $DB;

        $dbman = $DB->get_manager();
        $table = new xmldb_table('local_rwth_sd_sync_testtrmn');
        $dbman->drop_table($table);
        $table = new xmldb_table('local_rwth_sd_sync_testkurs');
        $dbman->drop_table($table);

        self::$courses = null;
        self::$appointments = null;

        ini_set('error_log', $this->oldlog);
    }

    protected function reset_local_rwth_sd_sync() {
        global $DB;

        $DB->delete_records('local_rwth_sd_sync_testtrmn', array());
        $DB->delete_records('local_rwth_sd_sync_testkurs', array());
    }

    /**
     * 
     */
    public function test_sync_courses() {
        global $DB, $CFG;
        
        //require_once($CFG->dirroot . "/local/rwth_sd_sync/classes/sdapi.php");
        
//        $trace = new text_progress_trace();
//        var_dump($USER);
        //$trace->output();
//        var_dump($CFG->mnet_localhost_id);
        
        $this->init_local_rwth_sd_sync();// init in erster testmethode: testreihenfolge == @depends klauseln
        
        $this->resetAfterTest(false);// true bei letztem test in datei ? testreihenfolge == @depends klauseln
        $this->preventResetByRollback();
        $this->reset_local_rwth_sd_sync();
        
        $trace = new null_progress_trace();
//        $trace = new text_progress_trace();// makes the test risky, as buffer not closed perhaps?

        //        use local_rwth_sd_sync\sdapi;
        // init plugin object
        // ACHTUNG: config now locally set in plugin object, global config no longer valid for the plugin object !
        $sdapi = new local_rwth_sd_sync\sdapi();
        
        // Test basic course sync.
        
        $sdapi->set_config('localcoursefield', 'idnumber', 'local_rwth_sd_sync');
        $sdapi->set_config('localcategoryfield', 'id', 'local_rwth_sd_sync');
        //set_config('defaultcategory', 1, 'local_rwth_sd_sync');
        
        $coursecat = $this->getDataGenerator()->create_category(array('name' => 'Test category 1', 'idnumber' => 'tcid1'));
        //set_config('newcoursecategory', 'category', 'local_rwth_sd_sync');

        $defcat = $DB->get_record('course_categories', array('id' => get_config('local_rwth_sd_sync','defaultcategory')));

        $course1 = array('fullname' => 'New course 1', 'shortname' => 'nc1', 'idnumber' => 'ncid1', 'category' => $coursecat->id);
        $course2 = array('fullname' => 'New course 2', 'shortname' => 'nc2', 'idnumber' => 'ncid2', 'category' => null);
        // Duplicate records are to be ignored.
        $course3 = array('fullname' => 'New course 3', 'shortname' => 'xx', 'idnumber' => 'yy2', 'category' => $coursecat->id);
        $course4 = array('fullname' => 'New course 4', 'shortname' => 'xx', 'idnumber' => 'yy3', 'category' => $coursecat->id);
        $course5 = array('fullname' => 'New course 5', 'shortname' => 'xx1', 'idnumber' => 'yy', 'category' => $coursecat->id);
        $course6 = array('fullname' => 'New course 6', 'shortname' => 'xx2', 'idnumber' => 'yy', 'category' => $coursecat->id);
        $numValidCourses = 4; // 2 duplicate

        $DB->insert_record('local_rwth_sd_sync_testkurs', $course1);
        $DB->insert_record('local_rwth_sd_sync_testkurs', $course2);
        $DB->insert_record('local_rwth_sd_sync_testkurs', $course3);
        $DB->insert_record('local_rwth_sd_sync_testkurs', $course4);
        $DB->insert_record('local_rwth_sd_sync_testkurs', $course5);
        $DB->insert_record('local_rwth_sd_sync_testkurs', $course6);

        // courses from local course list plus one phpunit standard course
        $this->assertEquals(1+count(self::$courses), $DB->count_records('course'));

        $result = 0;
        
        $result = $sdapi->sync_courses($trace);
        $trace->output('sync_courses return value: '.$result);
        
        // 1 phpunit standard kurs, 4 local courses, $numValidCourses synced kurse
        $this->assertEquals($numValidCourses + 1 + count(self::$courses), $DB->count_records('course'));

        $this->assertTrue($DB->record_exists('course', $course1));
        $course2['category'] = $defcat->id;
        $this->assertTrue($DB->record_exists('course', $course2));

        // People should NOT push duplicates there because the results are UNDEFINED! But anyway skip the duplicates.

        $this->assertEquals(1, $DB->count_records('course', array('idnumber' => 'yy')));
        $this->assertEquals(1, $DB->count_records('course', array('shortname' => 'xx')));

        // Check default number of sections matches with the created course sections.

        $recordcourse1 = $DB->get_record('course', $course1);
        $courseconfig = get_config('moodlecourse');
        $numsections = $DB->count_records('course_sections', array('course' => $recordcourse1->id));
        // To compare numsections we have to add topic 0 to default numsections.
        $this->assertEquals(($courseconfig->numsections + 1), $numsections);

        // Test category mapping via idnumber.

        $sdapi->set_config('localcategoryfield', 'idnumber', 'local_rwth_sd_sync');
        $course7 = array('fullname' => 'New course 7', 'shortname' => 'nc7', 'idnumber' => 'ncid7', 'category' => 'tcid1');
        $DB->insert_record('local_rwth_sd_sync_testkurs', $course7);
        
        $result = $sdapi->sync_courses($trace);
        $trace->output('sync_courses return value: '.$result);
       
//        var_dump($DB->count_records('local_rwth_sd_sync_testkurs'));
 //       $records = $DB->get_records('local_rwth_sd_sync_testkurs');
  //      var_dump($records);
        /*        $recordcourse1 = $DB->get_record('course', $course1);
         var_dump($recordcourse1);
         $recordcourse2 = $DB->get_record('course', $course2);
         var_dump($recordcourse2);
         */
    //    $records = $DB->get_records('course');
      //  var_dump($records);
        
        $this->assertEquals(1+$numValidCourses+1+count(self::$courses), $DB->count_records('course'));
        $this->assertTrue($DB->record_exists('course', $course1));
        $this->assertTrue($DB->record_exists('course', $course2));
        $course7['category'] = $coursecat->id;
        $this->assertTrue($DB->record_exists('course', $course7));

        // Test course template.

        $template = $this->getDataGenerator()->create_course(array('numsections' => 666, 'shortname' => 'crstempl'));
        $sdapi->set_config('templatecourse', 'crstempl', 'local_rwth_sd_sync');

        $course8 = array('fullname' => 'New course 8', 'shortname' => 'nc8', 'idnumber' => 'ncid8', 'category' => null);
        $DB->insert_record('local_rwth_sd_sync_testkurs', $course8);
        $result = $sdapi->sync_courses($trace);
        $trace->output('sync_courses return value: '.$result);
        
        $this->assertEquals(2+1+$numValidCourses+1+count(self::$courses), $DB->count_records('course'));
        $course8['category'] = $defcat->id;
        $record = $DB->get_record('course', $course8);
        $this->assertFalse(empty($record));
        $this->assertEquals(666, course_get_format($record)->get_last_section_number());
        
        // Test invalid category.

        $course9 = array('fullname' => 'New course 9', 'shortname' => 'nc9', 'idnumber' => 'ncid9', 'category' => 'xxxxxxx');
        $DB->insert_record('local_rwth_sd_sync_testkurs', $course9);
        $result = $sdapi->sync_courses($trace);
        $trace->output('sync_courses return value: '.$result);
        
        $this->assertEquals(2+1+$numValidCourses+1+count(self::$courses), $DB->count_records('course'));
        $this->assertFalse($DB->record_exists('course', array('idnumber' => 'ncid9')));

        // Test when categories not specified.

        $sdapi->set_config('newcoursecategory', '', 'local_rwth_sd_sync');
        $result = $sdapi->sync_courses($trace);
        $trace->output('sync_courses return value: '.$result);
        
        $this->assertEquals(1+2+1+$numValidCourses+1+count(self::$courses), $DB->count_records('course'));
        $this->assertTrue($DB->record_exists('course', array('idnumber' => 'ncid9')));

        // Test changed title and description.
        
        $this->assertTrue($DB->record_exists('local_rwth_sd_sync_testkurs', $course1));
        unset($course2['category']);
        $this->assertTrue($DB->record_exists('local_rwth_sd_sync_testkurs', $course2));
        
        $changedname = 'changed name course 1';
        $changeddesc = 'changed description course 2';

        $sdcourse1 = $DB->get_record('local_rwth_sd_sync_testkurs', $course1);
        $sdcourse1->fullname = $changedname;
        $DB->update_record('local_rwth_sd_sync_testkurs', $sdcourse1);
        
        $sdcourse2 = $DB->get_record('local_rwth_sd_sync_testkurs', $course2);
        
        $sdcourse2->description = $changeddesc;
        $DB->update_record('local_rwth_sd_sync_testkurs', $sdcourse2);

        $result = $sdapi->sync_courses($trace);
        $trace->output('sync_courses return value: '.$result);
        
        // number is still the same, update does not remove or add courses
        $this->assertEquals(1+2+1+$numValidCourses+1+count(self::$courses), $DB->count_records('course'));
        $this->assertTrue($DB->record_exists('course', array('idnumber' => $course1['idnumber'])));
        $this->assertTrue($DB->record_exists('course', array('idnumber' => $course2['idnumber'])));

        $recordcourse1 = $DB->get_record('course', array('idnumber' => $course1['idnumber']));
        $this->assertEquals($changedname, $recordcourse1->fullname);
        
        $recordcourse2 = $DB->get_record('course', array('idnumber' => $course2['idnumber']));
        $this->assertEquals($changeddesc, $recordcourse2->summary);
        
        
        // Final cleanup - remove extra tables, fixtures and caches.
        //$this->cleanup_local_rwth_sd_sync();// not yet, there is another test
        
        $trace->finished();
    }
    
    /**
     * @depends test_sync_courses
     */
    public function test_sync_termine() {
        global $DB, $CFG;
        
        //require_once($CFG->dirroot . "/local/rwth_sd_sync/classes/sdapi.php");
        
        //        $trace = new text_progress_trace();
        //        var_dump($USER);
        //$trace->output();
        //        var_dump($CFG->mnet_localhost_id);
        
        $this->resetAfterTest(true);// true bei letztem test in datei ? testreihenfolge == @depends klauseln
        $this->preventResetByRollback();
        $this->reset_local_rwth_sd_sync();
        
        $trace = new null_progress_trace();
        //$trace = new text_progress_trace();// makes the test risky, as buffer not closed perhaps?
        
        //        use local_rwth_sd_sync\sdapi;
        // init plugin object
        // ACHTUNG: config now locally set in plugin object, global config no longer valid for the plugin object !
        $sdapi = new local_rwth_sd_sync\sdapi();
        
        
/* moodle event format        
            self::$appointments[$i] = $this->getDataGenerator()->create_event(array(
            'name' => 'eventname'.$i, 
            'eventtype' => 'course', 
            'courseid' => $course1id, 
            'timestart' => $currenttime, 
            'timeduration' => $duration, 
            'description' => 'room'.$i));
  */      
        
        $course1 = self::$courses[1];
        $course1id = $course1->id;
        $course1idnumber = $course1->idnumber;
        
        $course2 = self::$courses[2];
        $course2id = $course2->id;
        $course2idnumber = $course2->idnumber;
        
        $course3 = self::$courses[3];
        $course3id = $course3->id;
        $course3idnumber = $course3->idnumber;
        
        date_default_timezone_set("Europe/Berlin");
        //strftime('%F %T', $currenttime)
        
        $currenttime = time();
        $duration = 3600;// 1 hour
        $gap = 7200;
        
        // SD PK: start, end, room -> no corresponding duplicates, PK moodle test db: id
        // assignments: name -> keyword, description -> room (with prefix and notes evtl.), timestart -> start, timeduration -> end - start
        $termin1 = array('courseid' => $course1idnumber, 'room' => 'dummyroom1', 'start' => strftime('%F %T', $currenttime), 'end' => strftime('%F %T', $currenttime + $duration), 'keyword' => 'keyword1');
        $termin2 = array('courseid' => $course1idnumber, 'room' => 'dummyroom1', 'start' => strftime('%F %T', $currenttime + $gap), 'end' => strftime('%F %T', $currenttime + $gap + $duration), 'keyword' => 'keyword1');
        $termin3 = array('courseid' => $course1idnumber, 'room' => 'dummyroom1', 'start' => strftime('%F %T', $currenttime), 'end' => strftime('%F %T', $currenttime - $duration), 'keyword' => 'keyword1');// invalid duration
        $termin4 = array('courseid' => $course1idnumber, 'room' => 'dummyroom1', 'start' => strftime('%F %T', $currenttime), 'end' => strftime('%F %T', $currenttime), 'keyword' => 'keyword1');
        $termin5 = array('courseid' => $course2idnumber, 'room' => 'dummyroom1', 'start' => strftime('%F %T', $currenttime), 'end' => strftime('%F %T', $currenttime + $duration), 'keyword' => 'keyword1');
        $termin6 = array('courseid' => $course1idnumber, 'room' => 'dummyroom2', 'start' => strftime('%F %T', $currenttime), 'end' => strftime('%F %T', $currenttime + $duration), 'keyword' => 'keyword1');
        $termin7 = array('courseid' => $course1idnumber, 'room' => 'dummyroom1', 'start' => strftime('%F %T', $currenttime), 'end' => strftime('%F %T', $currenttime + $duration), 'keyword' => 'keyword2');
        $termin8 = array('courseid' => $course1idnumber, 'room' => 'dummyroom1', 'start' => strftime('%F %T', $currenttime), 'end' => strftime('%F %T', $currenttime + $duration), 'keyword' => 'keyword1');// duplicate
        $termin9 = array('courseid' => null, 'room' => 'dummyroom1', 'start' => strftime('%F %T', $currenttime), 'end' => strftime('%F %T', $currenttime + $duration), 'keyword' => 'keyword1');// invalid course
        $termin10 = array('courseid' => $course1idnumber, 'room' => 'dummyroom3', 'start' => null, 'end' => strftime('%F %T', $currenttime + $duration), 'keyword' => 'keyword1');// invalid start
        $termin11 = array('courseid' => $course1idnumber, 'room' => 'dummyroom4', 'start' => strftime('%F %T', $currenttime), 'end' => null, 'keyword' => 'keyword1');// invalid end
        $termin12 = array('courseid' => $course3idnumber, 'room' => 'dummyroom4', 'start' => '0000-00-00 00:00:00', 'end' => '0000-00-00 00:00:00', 'keyword' => 'keyword1');// default SD time, valid record
        
        $numValidTermine = 7;// 1 duplicate, 1 without course, 2 without time information, 1 with invalid duration
        
        $DB->insert_record('local_rwth_sd_sync_testtrmn', $termin1);
        $DB->insert_record('local_rwth_sd_sync_testtrmn', $termin2);
        $DB->insert_record('local_rwth_sd_sync_testtrmn', $termin3);
        $DB->insert_record('local_rwth_sd_sync_testtrmn', $termin4);
        $DB->insert_record('local_rwth_sd_sync_testtrmn', $termin5);
        $DB->insert_record('local_rwth_sd_sync_testtrmn', $termin6);
        $DB->insert_record('local_rwth_sd_sync_testtrmn', $termin7);
        $DB->insert_record('local_rwth_sd_sync_testtrmn', $termin8);
        $DB->insert_record('local_rwth_sd_sync_testtrmn', $termin9);
        $DB->insert_record('local_rwth_sd_sync_testtrmn', $termin10);
        $DB->insert_record('local_rwth_sd_sync_testtrmn', $termin11);
        $DB->insert_record('local_rwth_sd_sync_testtrmn', $termin12);
        
        // appointments from local appointment list
        $this->assertEquals(count(self::$appointments), $DB->count_records('event'));

        
//         $trace->output('moodle events--------------------------------------- ');
        
//         var_dump($DB->count_records('event'));
//         $records = $DB->get_records('event');
//         var_dump($records);
        
        $result = 0;
        
        $result = $sdapi->sync_appointments($trace);
        $trace->output('sync_courses return value: '.$result);

//         $trace->output('moodle synced events--------------------------------------- ');
        
//         var_dump($DB->count_records('event'));
//         $records = $DB->get_records('event');
//         var_dump($records);
//        $recordtermin = $DB->get_record('event', array('courseid' => $course3id));//, 'timestart' => strtotime('0000-00-00 00:00:00'), 'timeduration' => 0));
//        var_dump($recordtermin);
        
        
        // 10 local appointments, $numValidTermine synced appointments
        $this->assertEquals($numValidTermine + count(self::$appointments), $DB->count_records('event'));
        
        // no textfield compariosn allowed ?! //, 'description' => $termin1['room'], 'name' => $termin1['keyword']
        //$this->assertTrue($DB->record_exists('event', array('courseid' => $course1id, 'timestart' => $currenttime, 'timeduration' => $duration)));
        $countrecords = $DB->count_records('event', array('courseid' => $course1id, 'timestart' => $currenttime, 'timeduration' => $duration));
        $this->assertEquals(3, $countrecords);

        $countrecords = $DB->count_records('event', array('courseid' => $course2id, 'timestart' => $currenttime, 'timeduration' => $duration));
        $this->assertEquals(1, $countrecords);

        $countrecords = $DB->count_records('event', array('courseid' => $course2id, 'timestart' => $currenttime, 'timeduration' => $duration));
        $this->assertEquals(1, $countrecords);
        
        $countrecords = $DB->count_records('event', array('courseid' => $course1id, 'timestart' => $currenttime + $gap, 'timeduration' => $duration));
        $this->assertEquals(1, $countrecords);
        
        $countrecords = $DB->count_records('event', array('courseid' => $course1id, 'timestart' => $currenttime, 'timeduration' => 0));
        $this->assertEquals(1, $countrecords);

        $countrecords = $DB->count_records('event', array('courseid' => $course3id, 'timestart' => strtotime('0000-00-00 00:00:00'), 'timeduration' => 0));
        $this->assertEquals(1, $countrecords);
        
        // Final cleanup - remove extra tables, fixtures and caches.
        $this->cleanup_local_rwth_sd_sync();
        
        $trace->finished();
    }
}
